@@ -15,11 +15,13 @@
 import copy
 import datetime
 import logging
+import pytz
 import six
 
 import mock
 from oslo_context import context
 from oslo_serialization import jsonutils
+from oslo_utils import timeutils
 from testtools import matchers
 
 from oslo_versionedobjects import base
@@ -57,6 +59,7 @@ class MyObj(base.VersionedObject, base.VersionedObjectDictCompat):
               'rel_objects': fields.ListOfObjectsField('MyOwnedObject',
                                                        nullable=True),
               'mutable_default': fields.ListOfStringsField(default=[]),
+              'timestamp': fields.DateTimeField(nullable=True),
               }
 
     @staticmethod
@@ -388,6 +391,15 @@ class TestDoSubobjectBackport(test.TestCase):
                                                 version,
                                                 version_manifest=manifest)
 
+    def test_do_subobject_backport_with_manifest_old_parent(self):
+        child = self.ChildObj(foo=1)
+        parent = self.ParentObj(child=child)
+        manifest = {'ChildObj': '1.0'}
+        parent_primitive = parent.obj_to_primitive(target_version='1.1',
+                                                   version_manifest=manifest)
+        child_primitive = parent_primitive['versioned_object.data']['child']
+        self.assertEqual('1.0', child_primitive['versioned_object.version'])
+
     def test_do_subobject_backport_list_object(self):
         child = self.ChildObj(foo=1)
         parent = self.ParentObjList(objects=[child])
@@ -403,6 +415,16 @@ class TestDoSubobjectBackport(test.TestCase):
                                                 version,
                                                 version_manifest=None)
 
+    def test_do_subobject_backport_list_object_with_manifest(self):
+        child = self.ChildObj(foo=1)
+        parent = self.ParentObjList(objects=[child])
+        manifest = {'ChildObj': '1.0', 'ParentObjList': '1.0'}
+        parent_primitive = parent.obj_to_primitive(target_version='1.0',
+                                                   version_manifest=manifest)
+        self.assertEqual('1.0', parent_primitive['versioned_object.version'])
+        child_primitive = parent_primitive['versioned_object.data']['objects']
+        self.assertEqual('1.0', child_primitive[0]['versioned_object.version'])
+
     def test_do_subobject_backport_null_child(self):
         parent = self.ParentObj(child=None)
         parent_primitive = parent.obj_to_primitive()['versioned_object.data']
@@ -417,6 +439,13 @@ class TestDoSubobjectBackport(test.TestCase):
                              "have been called because the subobject is "
                              "None.")
 
+    def test_to_primitive_calls_make_compatible_manifest(self):
+        obj = self.ParentObj()
+        with mock.patch.object(obj, 'obj_make_compatible_from_manifest') as m:
+            obj.obj_to_primitive(target_version='1.0',
+                                 version_manifest=mock.sentinel.manifest)
+            m.assert_called_once_with(mock.ANY, '1.0', mock.sentinel.manifest)
+
 
 def compare_obj(test, obj, db_obj, subs=None, allow_missing=None,
                 comparators=None):
@@ -548,7 +577,7 @@ class TestFixture(_BaseTestCase):
         hashes = checker.get_hashes()
         # NOTE(danms): If this object's version or hash changes, this needs
         # to change. Otherwise, leave it alone.
-        self.assertEqual('1.6-7157ceb869f8f63fb9a955e6a7080ad7',
+        self.assertEqual('1.6-fb5f5379168bf08f7f2ce0a745e91027',
                          hashes['TestSubclassedObject'])
 
     def test_test_hashes(self):
@@ -944,10 +973,10 @@ class _TestObject(object):
         self.assertEqual(obj.bar, 'updated')
 
     def test_contains(self):
-        obj = MyObj()
-        self.assertNotIn('foo', obj)
-        obj.foo = 1
-        self.assertIn('foo', obj)
+        obj = MyOwnedObject()
+        self.assertNotIn('baz', obj)
+        obj.baz = 1
+        self.assertIn('baz', obj)
         self.assertNotIn('does_not_exist', obj)
 
     def test_obj_attr_is_set(self):
@@ -994,7 +1023,7 @@ class _TestObject(object):
         base_fields = []
         myobj_fields = (['foo', 'bar', 'missing',
                          'readonly', 'rel_object',
-                         'rel_objects', 'mutable_default'] +
+                         'rel_objects', 'mutable_default', 'timestamp'] +
                         base_fields)
         myobj3_fields = ['new_field']
         self.assertTrue(issubclass(TestSubclassedObject, MyObj))
@@ -1030,6 +1059,25 @@ class _TestObject(object):
         obj.obj_reset_changes()
         self.assertEqual({}, obj.obj_get_changes())
 
+        timestamp = datetime.datetime(2001, 1, 1, tzinfo=pytz.utc)
+        with mock.patch.object(timeutils, 'utcnow') as mock_utcnow:
+            mock_utcnow.return_value = timestamp
+            obj.timestamp = timeutils.utcnow()
+            self.assertEqual({'timestamp': timestamp}, obj.obj_get_changes())
+
+        obj.obj_reset_changes()
+        self.assertEqual({}, obj.obj_get_changes())
+
+        # Timestamp without tzinfo causes mismatch
+        timestamp = datetime.datetime(2001, 1, 1)
+        with mock.patch.object(timeutils, 'utcnow') as mock_utcnow:
+            mock_utcnow.return_value = timestamp
+            obj.timestamp = timeutils.utcnow()
+            self.assertRaises(TypeError, obj.obj_get_changes())
+
+        obj.obj_reset_changes()
+        self.assertEqual({}, obj.obj_get_changes())
+
     def test_obj_fields(self):
         class TestObj(base.VersionedObject):
             fields = {'foo': fields.Field(fields.Integer())}
@@ -1082,7 +1130,7 @@ class _TestObject(object):
         obj = MyObj(foo=123)
         self.assertEqual('MyObj(bar=<?>,foo=123,missing=<?>,'
                          'mutable_default=<?>,readonly=<?>,'
-                         'rel_object=<?>,rel_objects=<?>)',
+                         'rel_object=<?>,rel_objects=<?>,timestamp=<?>)',
                          repr(obj))
 
     def test_obj_make_obj_compatible_with_relationships(self):
